//#ifndef __CINT__
#include "RooGlobalFunc.h"
//#endif
#include "RooRealVar.h"
#include "RooArgList.h"
#include "RooFormulaVar.h"
#include "RooDataSet.h"
#include "RooGaussian.h"
#include "RooChebychev.h"
#include "RooExponential.h"
#include "RooBifurGauss.h"
#include "RooAddModel.h"
#include "RooProdPdf.h"
#include "TCanvas.h"
#include "RooPlot.h"
#include "RooHist.h"
#include "RooCBShape.h"
#include "RooPolynomial.h"
#include "RooBinning.h"
#include "RooPolyVar.h"
#include "TH1.h"
#include "TH2.h"
#include "RooAddPdf.h"
#include "RooProdPdf.h"
#include "RooFitResult.h"
#include "RooGenericPdf.h"
#include "RooLandau.h"
#include "TChain.h"
#include "TCanvas.h"
#include "TAxis.h"
#include "RooPlot.h"
#include "RooCategory.h"
#include "RooSuperCategory.h"
#include "RooSimultaneous.h"
#include "RooNLLVar.h"
#include "TFile.h"
#include "myRooJohnsonSU.cpp"
#include "myRooPolBG.cpp"

using namespace RooFit ;
using namespace std;

void kk_2dfit_sig()
{
  TFile *f = new TFile("required_for_2dfit.root");
  TTree *tree = (TTree*)f->Get("DstD0PiKK");
  RooRealVar x("DzPi_M","", 2.005, 2.03);//d0pi
  RooRealVar y("Dz_M","", 1.75, 2.05);//d0
  RooDataSet* dataxy = new RooDataSet("data","data",tree,RooArgSet(x,y)); 

  /*-----------------------S I G N A L----------------------*/

  //d0pi
  RooRealVar f_J_sig("f_{J}", "",0.82,0, 1);                                                                                                 
  RooRealVar mu_J_sig("#mu_{J}","mean_johnson",2.01,2.006, 2.03);                                                          
  RooRealVar sigma_J_sig("#sigma_{J}", "sigma_johnson",  0.01, 0, 0.5);                                                                       
  RooRealVar gamma_J_sig("#gamma_{J}", "gamma", 1, -20., 20.);                                                                                
  RooRealVar delta_J_sig("#delta_{J}", "delta", 10, -20., 20.);                                                                               
  myRooJohnsonSU johnson_sig("johnson", "Johnson PDF", x, mu_J_sig, sigma_J_sig, delta_J_sig, gamma_J_sig);                                  

  RooRealVar f_G1_sig("f_{G}", "",0.38, 0, 1);                                                                                               
  RooRealVar mu_G1_sig("#mu_{G1}", "mean_gauss1",2.01,2.00, 2.03);                                                         
  RooRealVar mu_G2_sig("#mu_{G2}", "mean_gauss2", 2.01, 2.00, 2.03);                                                         
  RooRealVar sigma_G1_sig("#sigma_{G1}", "sigma_gauss1",  0.01,0, 0.5);                                                                       
  RooRealVar sigma_G2_sig("#sigma_{G2}", "sigma_gauss2",  0.01, 0, 0.5);
  RooGaussian gauss1_sig("gauss1", "1st gaussian PDF", x, mu_G1_sig, sigma_G1_sig);                                                           
  RooGaussian gauss2_sig("gauss2", "2nd gaussian PDF", x, mu_G1_sig, sigma_G2_sig);                                                           
  RooAddPdf doublegaussian_sig("doublegaussian", "double_gaussian", gauss1_sig, gauss2_sig, f_G1_sig);                                        
  RooAddPdf sig_d0pi("sig_d0pi", "signal_function", johnson_sig, doublegaussian_sig, f_J_sig);

  //kk
  RooRealVar f_J_sig_d0("f_{J}", "",0.48,0, 1);                                                                                               
  RooRealVar mu_J_sig_d0("#mu_{J}","mean_johnson",1.863, 1.83, 2.03);                                                          
  RooRealVar sigma_J_sig_d0("#sigma_{J}", "sigma_johnson",  0.011, 0, 10);                                                                    
  RooRealVar gamma_J_sig_d0("#gamma_{J}", "gamma", 1, -20., 20.);                                                                             
  RooRealVar delta_J_sig_d0("#delta_{J}", "delta", 10, -20., 20.);                                                                            
  myRooJohnsonSU johnson_sig_d0("johnson", "Johnson PDF", y, mu_J_sig_d0, sigma_J_sig_d0, delta_J_sig_d0, gamma_J_sig_d0 );

  RooRealVar f_G1_sig_d0("f_{G}", "",0.26, 0, 1);                                                                                            
  RooRealVar mu_G1_sig_d0("#mu_{G1}", "mean_gauss1",1.865, 1.858, 2.04);                                                         
  RooRealVar mu_G2_sig_d0("#mu_{G2}", "mean_gauss2", 1.863, 1.82, 2.03);                                                         
  RooRealVar sigma_G1_sig_d0("#sigma_{G1}", "sigma_gauss1",  0.01,0, 10);                                                                     
  RooRealVar sigma_G2_sig_d0("#sigma_{G2}", "sigma_gauss2",  0.01, 0, 10);
  RooGaussian gauss1_sig_d0("gauss1", "1st gaussian PDF", y, mu_G1_sig_d0, sigma_G1_sig_d0);                                                  
  RooGaussian gauss2_sig_d0("gauss2", "2nd gaussian PDF", y, mu_G1_sig_d0, sigma_G2_sig_d0);                                                  
  RooAddPdf doublegaussian_sig_d0("doublegaussian", "double_gaussian", gauss1_sig_d0, gauss2_sig_d0, f_G1_sig_d0);                            
  RooAddPdf sig_kk("sig_kk", "signal_function", johnson_sig_d0, doublegaussian_sig_d0, f_J_sig_d0);

  //signal peak
  RooProdPdf signal_peak("signal peak", "signal_function", sig_d0pi, sig_kk);

  //random pion (common for all)
  RooRealVar threshold("threshold", "", 2.00441);
  RooRealVar alpha("#alpha", "alpha", 20, -100., 100.);
  RooRealVar beta("#beta", "beta", 0.);
  myRooPolBG bkg("bkg","background component", x, threshold, alpha, beta);

  //signal & random pion
  RooProdPdf signal("signal", "signal_function", sig_kk, bkg);


  /*------------------------C O M B I N A T O R I A L----------------------------*/
  RooRealVar c1("c", "c", -50.,-100.,0.);
  RooExponential comb1("combinatorial", "", y, c1);
  RooProdPdf comb_bkg("comb", "exponential in m(kk) x random pi", comb1, bkg);
  
  /*------------------------Y I E L D S  F O R  A L L  C O M P O N E N T S----------------------------*/
  RooRealVar sig_peak_yield("","",3830,300,80000);
  RooRealVar sig_randompi_yield("","",720,200,30000);

  RooRealVar comb_yield("","",720,200,30000);
  
  /*------------------------C O M P L E T E  M O D E L----------------------------*/

  RooAddPdf model("model","",RooArgList(signal_peak,signal, comb_bkg), RooArgList(sig_peak_yield,sig_randompi_yield, comb_yield));

  /*----------------------------------- FIT THE DATA-------------------------------------*/
  RooFitResult *fitresult = model.fitTo(*dataxy, Save(true), Strategy(2),Extended(true));
  //model.fitTo(*dataxy);
  
  /*-----------------------------------PLOT THE RESULTS-------------------------------------*/
  //Plot x distribution of data and projection of model on x
  RooPlot *xframe = x.frame();
  dataxy->plotOn(xframe);
  model.plotOn(xframe);
  
  //Plot y distribution of data and projection of model on y
  RooPlot *yframe = y.frame();
  dataxy->plotOn(yframe);
  model.plotOn(yframe);
    
  // Make two-dimensional plot in x vs y
  TH1 *hh_model = model.createHistogram("hh_model", x, Binning(100), YVar(y, Binning(100)));
  hh_model->SetLineColor(kBlue);
  
  // Make canvas and draw RooPlots
  TCanvas *c = new TCanvas("c", "c", 1600, 400);
  c->Divide(3);
  c->cd(1);
  gPad->SetLeftMargin(0.15);
  xframe->GetYaxis()->SetTitleOffset(1.6);
  xframe->Draw();
  c->cd(2);
  gPad->SetLeftMargin(0.15);
  yframe->GetYaxis()->SetTitleOffset(1.6);
  yframe->Draw();
  c->cd(3);
  gPad->SetLeftMargin(0.20);
  hh_model->GetZaxis()->SetTitleOffset(2.5);
  hh_model->Draw("surf");

  
}
